import os
from typing import TYPE_CHECKING

from aviary.core import TASK_DATASET_REGISTRY
from aviary.envs.labbench import PaperQATaskDataset

from .env import LFRQAPairwiseEvalEnv, LFRQAQuestion

if TYPE_CHECKING:
    import pandas as pd


def read_csv(path: str | os.PathLike) -> "pd.DataFrame":
    try:
        import pandas as pd
    except ImportError as exc:
        raise ImportError(
            "Reading in a CSV requires the 'csv' extra for 'pandas'. Please:"
            " `pip install aviary.lfrqa[csv]`."
        ) from exc
    return pd.read_csv(path)


class LFRQATaskDataset(PaperQATaskDataset[LFRQAPairwiseEvalEnv]):
    """Task dataset for custom evaluation of non-multiple choice questions."""

    def __init__(self, data: str | os.PathLike | list[LFRQAQuestion], **kwargs):
        super().__init__(**kwargs)
        self.data: "list[LFRQAQuestion] | pd.DataFrame" = (  # noqa: UP037
            data if isinstance(data, list) else read_csv(data)
        )

    def _make_query(self, idx: int) -> LFRQAQuestion:
        if isinstance(self.data, list):
            return self.data[idx]
        return LFRQAQuestion(  # type: ignore[call-arg]
            qid=self.data.iloc[idx].qid,
            question=self.data.iloc[idx].question,
            answer=self.data.iloc[idx].answer,
            gold_doc_ids=self.data.iloc[idx].gold_doc_ids,
            **(self._question_kwargs or {}),
        )

    def get_new_env_by_idx(self, idx: int) -> LFRQAPairwiseEvalEnv:
        """Create a new environment instance for the given index."""
        return LFRQAPairwiseEvalEnv(
            query=self._make_query(idx),
            settings=self._settings,
            docs=self._base_docs.model_copy(),
            **self._env_kwargs,
        )

    def __len__(self) -> int:
        return len(self.data)


TASK_DATASET_NAME = "lfrqa"
TASK_DATASET_REGISTRY[TASK_DATASET_NAME] = (
    LFRQATaskDataset.__module__,
    LFRQATaskDataset.__name__,
)



import pathlib
from unittest.mock import MagicMock

import pandas as pd
import pytest
from aviary.core import Environment
from ldp.alg import MeanMetricsCallback

from aviary.envs.lfrqa.env import LFRQAPairwiseEvalEnv, LFRQAQuestion
from aviary.envs.lfrqa.task import LFRQATaskDataset

TESTS_DIR = pathlib.Path(__file__).parent
STUB_DATA_DIR = TESTS_DIR / "stub_data"
MINI_LFRQA_CSV = STUB_DATA_DIR / "mini_lfrqa.csv"


def test_availability() -> None:
    assert "lfrqa" in Environment.available()


@pytest.fixture(name="mini_lfrqa", scope="module")
def fixture_mini_lfrqa() -> list[LFRQAQuestion]:
    return [
        LFRQAQuestion(**row)  # type: ignore[misc]
        for row in pd.read_csv(MINI_LFRQA_CSV)[
            ["qid", "question", "answer", "gold_doc_ids"]
        ].to_dict(orient="records")
    ]


@pytest.mark.parametrize("test_csv_read", [False, True])
@pytest.mark.asyncio
async def test_env_construction(
    test_csv_read: bool, mini_lfrqa: list[LFRQAQuestion]
) -> None:
    spy_callback = MagicMock()
    dataset = LFRQATaskDataset(
        data=MINI_LFRQA_CSV if test_csv_read else mini_lfrqa,
        evaluation_callback=spy_callback,
    )
    MeanMetricsCallback(eval_dataset=dataset)  # Confirm we could use this
    assert len(dataset) == 4

    env = dataset.get_new_env_by_idx(0)  # noqa: FURB184
    assert isinstance(env, LFRQAPairwiseEvalEnv)
    assert await env.get_id() == "science-search-test-30"
    assert env._evaluation_callback == spy_callback
